package RokuCtrl;

  # RokuCtrl.pm
  # ver. 0.1
  # Copyright 2008 inutch  http://d.hatena.ne.jp/inutch/
  #
  # This program is free software: you can redistribute it and/or modify
  # it under the terms of the GNU Lesser General Public License as published by
  # the Free Software Foundation, either version 3 of the License, or
  # (at your option) any later version.
  #
  # This program is distributed in the hope that it will be useful,
  # but WITHOUT ANY WARRANTY; without even the implied warranty of
  # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  # GNU Lesser General Public License for more details.
  #
  # You should have received a copy of the GNU Lesser General Public License
  # along with this program.  If not, see <http://www.gnu.org/licenses/>.

use Net::Telnet;

sub new{
  my $class = shift;
  my ($host, $port) = @_;
  my $telnet = new Net::Telnet(
    Host => $host,
    Port => $port,
    Timeout => 30,
  );
  my $this = {
    "telnet" => $telnet
  };

  bless $this, $class;
  return $this;
}

sub open{
  my $this = shift;
  my $telnet = $this->{telnet};
  $telnet->open();
  $telnet->waitfor('/roku: ready/');
  $telnet->print("GetConnectedServer");
  $telnet->waitfor('/GetConnectedServer: OK/');
  $telnet->print("SetSongListSort albumTrack");
  $telnet->waitfor('/SetSongListSort: OK/');
}

sub close{
  my $this = shift;
  my $telnet = $this->{telnet};
  $telnet->close();
}

sub getartistlist{
  my $this = shift;
  my $telnet = $this->{telnet};
  my $result;
  my $tmp;
  my @list;
  $telnet->print("ListArtists");
  ($result) = $telnet->waitfor('/TransactionComplete|ListArtists: ListResultEnd/');
  if( $result =~ /ListArtists: GenericError/ ){
    return ();
  }
  chomp($result);
  @list = split(/\nListArtists: /, $result);
  $tmp = shift @list;
  while($tmp = shift @list){
    if( $tmp =~ /ListResultSize/ ){ last; }
  }
  return @list;
}

sub getalbumlist{
  my $this = shift;
  my $artist = shift;
  my $telnet = $this->{telnet};
  my $result;
  my $tmp;
  my @list;
  if( $artist and $artist ne "" ){
    $telnet->print("SetBrowseFilterArtist " . $artist);
    $telnet->waitfor('/SetBrowseFilterArtist: OK/');
  }
  $telnet->print("ListAlbums");
  ($result) = $telnet->waitfor('/TransactionComplete|ListAlbums: ListResultEnd/');
  if( $result =~ /GenericError/ ){
    return ();
  }
  chomp($result);
  @list = split(/\nListAlbums: /, $result);
  $tmp = shift @list;
  while($tmp = shift @list){
    if( $tmp =~ /ListResultSize/ ){ last; }
  }
  return @list;
}

sub getsonglist{
  my $this = shift;
  my ($artist, $album) = @_;
  my $telnet = $this->{telnet};
  my $result;
  my $tmp;
  my @list;
  if( $artist and $artist ne "" ){
    $telnet->print("SetBrowseFilterArtist " . $artist);
    $telnet->waitfor('/SetBrowseFilterArtist: OK/');
  }
  if( $album and $album ne "" ){
    $telnet->print("SetBrowseFilterAlbum " . $album);
    $telnet->waitfor('/SetBrowseFilterAlbum: OK/');
  }
  $telnet->print("ListSongs");
  ($result) = $telnet->waitfor('/TransactionComplete|ListSongs: ListResultEnd/');
  if( $result =~ /GenericError/ ){
    return ();
  }
  chomp($result);
  @list = split(/\nListSongs: /, $result);
  $tmp = shift @list;
  while($tmp = shift @list){
    if( $tmp =~ /ListResultSize/ ){ last; }
  }
  return @list;
}

sub filterandplay{
  my $this = shift;
  my ($artist, $album, $number) = @_;
  my $telnet = $this->{telnet};
  my $result;
  my $tmp;
  my @list;
  if( $artist and $artist ne "" ){
    $telnet->print("SetBrowseFilterArtist " . $artist);
    $telnet->waitfor('/SetBrowseFilterArtist: OK/');
  }
  if( $album and $album ne "" ){
    $telnet->print("SetBrowseFilterAlbum " . $album);
    $telnet->waitfor('/SetBrowseFilterAlbum: OK/');
  }
  $telnet->print("ListSongs");
  ($result) = $telnet->waitfor('/TransactionComplete|ListSongs: ListResultEnd/');
  if( $result =~ /GenericError/ ){
    return ();
  }
  chomp($result);
  @list = split(/\nListSongs: /, $result);
  $tmp = shift @list;
  while($tmp = shift @list){
    if( $tmp =~ /ListResultSize/ ){ last; }
  }
  $telnet->print("QueueAndPlay " . $number);
  ($result) = $telnet->waitfor('/QueueAndPlay: OK/');
  return @list;
}

1;